@@ -272,6 +272,9 @@ barinference <- function(obj, tab, phat, counts) {
         twoway <- TRUE
     }
 
+    # CI interval width:
+    alpha <- 1 - (1 - opts$ci.width) / 2
+
     lapply(inf.type, function(type) {
         switch(type,
             "conf" = {
@@ -295,7 +298,7 @@ barinference <- function(obj, tab, phat, counts) {
                                 cis <- apply(b$t, 2,
                                     function(x) {
                                         c(
-                                            quantile(x, probs = c(0.025, 0.975)),
+                                            quantile(x, probs = c(1 - alpha, alpha)),
                                             mean(x)
                                         )
                                     }
@@ -332,7 +335,7 @@ barinference <- function(obj, tab, phat, counts) {
                             cis <- apply(b$t, 2,
                                 function(x) {
                                     c(
-                                        quantile(x, probs = c(0.025, 0.975)),
+                                        quantile(x, probs = c(1 - alpha, alpha)),
                                         mean(x)
                                     )
                                 }
@@ -347,21 +350,26 @@ barinference <- function(obj, tab, phat, counts) {
                 } else {
                     if (svy) {
                         if (twoway) {
-                            est <- svyby(~x, by = ~y, obj$df,
+                            fit <- svyby(~x, by = ~y, obj$df,
                                 FUN = svymean,
-                                vartype = "ci",
                                 drop.empty.groups = FALSE,
                                 na.rm = TRUE
                             )
-                            est <- est[, -1]
+                            est <- coef(fit)
+                            ci <- confint(fit, level = opts$ci.width)
                             nc <- length(levels(obj$df$variables$x))
                             list(
-                                lower = as.matrix(est[, nc + 1:nc]),
-                                upper = as.matrix(est[, 2 * nc + 1:nc]),
-                                estimate = as.matrix(est[, 1:nc])
+                                lower = matrix(ci[, 1], ncol = nc),
+                                upper = matrix(ci[, 2], ncol = nc),
+                                estimate = matrix(est, ncol = nc)
                             )
                         } else {
-                            ci <- t(confint(svymean(~x, obj$df, na.rm = TRUE)))
+                            ci <- t(
+                                confint(
+                                    svymean(~x, obj$df, na.rm = TRUE),
+                                    level = opts$ci.width
+                                )
+                            )
                             list(
                                 lower = ci[1, , drop = FALSE],
                                 upper = ci[2, , drop = FALSE],
@@ -376,7 +384,7 @@ barinference <- function(obj, tab, phat, counts) {
                                     n <- sum(x)
                                     p <- ifelse(x >= opts$min.count, x / n, NA)
                                     se <- sqrt(p * (1 - p) / n)
-                                    se * 1.96
+                                    se * qnorm(alpha)
                                 }
                             )
                         )
@@ -503,6 +511,7 @@ barinference <- function(obj, tab, phat, counts) {
         ),
         na.rm = TRUE
     )
+    attr(result, "ci.width") <- opts$ci.width
 
     result
 }
